@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,15 +16,17 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include	"sfconfig.h"
+
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
+#include	<limits.h>
+#include	<math.h>
 
 #include	"sndfile.h"
-#include	"config.h"
 #include	"sfendian.h"
 #include	"common.h"
-#include	"float_cast.h"
 
 #if CPU_IS_LITTLE_ENDIAN
 	#define FLOAT32_READ	float32_le_read
@@ -56,30 +58,22 @@ static sf_count_t	host_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t	host_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t	host_read_f2d	(SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t	host_write_s2f	(SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t	host_write_i2f	(SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t	host_write_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t	host_write_d2f	(SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t	host_write_s2f	(SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t	host_write_i2f	(SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t	host_write_f	(SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t	host_write_d2f	(SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
-static	void	f2s_array 	(float *src, int count, short *dest) ;
-static	void	f2i_array 	(float *src, int count, int *dest) ;
-static	void	f2d_array 	(float *src, int count, double *dest) ;
-
-static 	void	s2f_array 	(short *src, float *dest, int count) ;
-static 	void	i2f_array 	(int *src, float *dest, int count) ;
-static 	void	d2f_array 	(double *src, float *dest, int count) ;
-
-static void		float32_peak_update	(SF_PRIVATE *psf, float *buffer, int count, int indx) ;
+static void		float32_peak_update	(SF_PRIVATE *psf, const float *buffer, int count, sf_count_t indx) ;
 
 static sf_count_t	replace_read_f2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
 static sf_count_t	replace_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t	replace_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t	replace_read_f2d	(SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t	replace_write_s2f	(SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t	replace_write_i2f	(SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t	replace_write_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t	replace_write_d2f	(SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t	replace_write_s2f	(SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t	replace_write_i2f	(SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t	replace_write_f	(SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t	replace_write_d2f	(SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
 static	void	bf2f_array (float *buffer, int count) ;
 static	void	f2bf_array (float *buffer, int count) ;
@@ -101,7 +95,7 @@ float32_init	(SF_PRIVATE *psf)
 	if (psf->mode == SFM_READ || psf->mode == SFM_RDWR)
 	{	switch (psf->endian + float_caps)
 		{	case (SF_ENDIAN_BIG + FLOAT_CAN_RW_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= host_read_f2s ;
 					psf->read_int		= host_read_f2i ;
 					psf->read_float		= host_read_f ;
@@ -109,7 +103,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_CAN_RW_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= host_read_f2s ;
 					psf->read_int		= host_read_f2i ;
 					psf->read_float		= host_read_f ;
@@ -117,7 +111,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + FLOAT_CAN_RW_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= host_read_f2s ;
 					psf->read_int		= host_read_f2i ;
 					psf->read_float		= host_read_f ;
@@ -125,7 +119,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_CAN_RW_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= host_read_f2s ;
 					psf->read_int		= host_read_f2i ;
 					psf->read_float		= host_read_f ;
@@ -134,7 +128,7 @@ float32_init	(SF_PRIVATE *psf)
 
 			/* When the CPU is not IEEE compatible. */
 			case (SF_ENDIAN_BIG + FLOAT_BROKEN_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= replace_read_f2s ;
 					psf->read_int		= replace_read_f2i ;
 					psf->read_float		= replace_read_f ;
@@ -142,7 +136,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_BROKEN_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= replace_read_f2s ;
 					psf->read_int		= replace_read_f2i ;
 					psf->read_float		= replace_read_f ;
@@ -150,7 +144,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + FLOAT_BROKEN_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->read_short		= replace_read_f2s ;
 					psf->read_int		= replace_read_f2i ;
 					psf->read_float		= replace_read_f ;
@@ -158,7 +152,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_BROKEN_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->read_short		= replace_read_f2s ;
 					psf->read_int		= replace_read_f2i ;
 					psf->read_float		= replace_read_f ;
@@ -172,7 +166,7 @@ float32_init	(SF_PRIVATE *psf)
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
 	{	switch (psf->endian + float_caps)
 		{	case (SF_ENDIAN_LITTLE + FLOAT_CAN_RW_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= host_write_s2f ;
 					psf->write_int		= host_write_i2f ;
 					psf->write_float	= host_write_f ;
@@ -180,7 +174,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + FLOAT_CAN_RW_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= host_write_s2f ;
 					psf->write_int		= host_write_i2f ;
 					psf->write_float	= host_write_f ;
@@ -188,7 +182,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + FLOAT_CAN_RW_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= host_write_s2f ;
 					psf->write_int		= host_write_i2f ;
 					psf->write_float	= host_write_f ;
@@ -196,7 +190,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_CAN_RW_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= host_write_s2f ;
 					psf->write_int		= host_write_i2f ;
 					psf->write_float	= host_write_f ;
@@ -205,7 +199,7 @@ float32_init	(SF_PRIVATE *psf)
 
 			/* When the CPU is not IEEE compatible. */
 			case (SF_ENDIAN_BIG + FLOAT_BROKEN_LE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= replace_write_s2f ;
 					psf->write_int		= replace_write_i2f ;
 					psf->write_float	= replace_write_f ;
@@ -213,7 +207,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_BROKEN_LE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= replace_write_s2f ;
 					psf->write_int		= replace_write_i2f ;
 					psf->write_float	= replace_write_f ;
@@ -221,7 +215,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_BIG + FLOAT_BROKEN_BE) :
-					psf->float_endswap = SF_FALSE ;
+					psf->data_endswap = SF_FALSE ;
 					psf->write_short	= replace_write_s2f ;
 					psf->write_int		= replace_write_i2f ;
 					psf->write_float	= replace_write_f ;
@@ -229,7 +223,7 @@ float32_init	(SF_PRIVATE *psf)
 					break ;
 
 			case (SF_ENDIAN_LITTLE + FLOAT_BROKEN_BE) :
-					psf->float_endswap = SF_TRUE ;
+					psf->data_endswap = SF_TRUE ;
 					psf->write_short	= replace_write_s2f ;
 					psf->write_int		= replace_write_i2f ;
 					psf->write_float	= replace_write_f ;
@@ -273,9 +267,9 @@ float32_be_read (unsigned char *cptr)
 		fvalue *= -1 ;
 
 	if (exponent > 0)
-		fvalue *= (1 << exponent) ;
+		fvalue *= pow (2.0, exponent) ;
 	else if (exponent < 0)
-		fvalue /= (1 << abs (exponent)) ;
+		fvalue /= pow (2.0, abs (exponent)) ;
 
 	return fvalue ;
 } /* float32_be_read */
@@ -301,9 +295,9 @@ float32_le_read (unsigned char *cptr)
 		fvalue *= -1 ;
 
 	if (exponent > 0)
-		fvalue *= (1 << exponent) ;
+		fvalue *= pow (2.0, exponent) ;
 	else if (exponent < 0)
-		fvalue /= (1 << abs (exponent)) ;
+		fvalue /= pow (2.0, abs (exponent)) ;
 
 	return fvalue ;
 } /* float32_le_read */
@@ -314,7 +308,7 @@ float32_le_write (float in, unsigned char *out)
 
 	memset (out, 0, sizeof (int)) ;
 
-	if (in == 0.0)
+	if (fabs (in) < 1e-30)
 		return ;
 
 	if (in < 0.0)
@@ -349,7 +343,7 @@ float32_be_write (float in, unsigned char *out)
 
 	memset (out, 0, sizeof (int)) ;
 
-	if (in == 0.0)
+	if (fabs (in) < 1e-30)
 		return ;
 
 	if (in < 0.0)
@@ -383,7 +377,7 @@ float32_be_write (float in, unsigned char *out)
 */
 
 static void
-float32_peak_update	(SF_PRIVATE *psf, float *buffer, int count, int indx)
+float32_peak_update	(SF_PRIVATE *psf, const float *buffer, int count, sf_count_t indx)
 {	int 	chan ;
 	int		k, position ;
 	float	fmaxval ;
@@ -397,9 +391,9 @@ float32_peak_update	(SF_PRIVATE *psf, float *buffer, int count, int indx)
 				position = k ;
 				} ;
 
-		if (fmaxval > psf->pchunk->peaks [chan].value)
-		{	psf->pchunk->peaks [chan].value = fmaxval ;
-			psf->pchunk->peaks [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
+		if (fmaxval > psf->peak_info->peaks [chan].value)
+		{	psf->peak_info->peaks [chan].value = fmaxval ;
+			psf->peak_info->peaks [chan].position = psf->write_current + indx + (position / psf->sf.channels) ;
 			} ;
 		} ;
 
@@ -432,29 +426,108 @@ float32_get_capability	(SF_PRIVATE *psf)
 	return (CPU_IS_LITTLE_ENDIAN) ? FLOAT_BROKEN_LE : FLOAT_BROKEN_BE ;
 } /* float32_get_capability */
 
+/*=======================================================================================
+*/
+
+static void
+f2s_array (const float *src, int count, short *dest, float scale)
+{
+	while (--count >= 0)
+	{	dest [count] = lrintf (scale * src [count]) ;
+		} ;
+} /* f2s_array */
+
+static void
+f2s_clip_array (const float *src, int count, short *dest, float scale)
+{	while (--count >= 0)
+	{	float tmp = scale * src [count] ;
+
+		if (CPU_CLIPS_POSITIVE == 0 && tmp > 32767.0)
+			dest [count] = SHRT_MAX ;
+		else if (CPU_CLIPS_NEGATIVE == 0 && tmp < -32768.0)
+			dest [count] = SHRT_MIN ;
+		else
+			dest [count] = lrintf (tmp) ;
+		} ;
+} /* f2s_clip_array */
+
+static inline void
+f2i_array (const float *src, int count, int *dest, float scale)
+{	while (--count >= 0)
+	{	dest [count] = lrintf (scale * src [count]) ;
+		} ;
+} /* f2i_array */
+
+static inline void
+f2i_clip_array (const float *src, int count, int *dest, float scale)
+{	while (--count >= 0)
+	{	float tmp = scale * src [count] ;
+
+		if (CPU_CLIPS_POSITIVE == 0 && tmp > (1.0 * INT_MAX))
+			dest [count] = INT_MAX ;
+		else if (CPU_CLIPS_NEGATIVE == 0 && tmp < (-1.0 * INT_MAX))
+			dest [count] = INT_MIN ;
+		else
+			dest [count] = lrintf (tmp) ;
+		} ;
+} /* f2i_clip_array */
+
+static inline void
+f2d_array (const float *src, int count, double *dest)
+{	while (--count >= 0)
+	{	dest [count] = src [count] ;
+		} ;
+} /* f2d_array */
+
+static inline void
+s2f_array (const short *src, float *dest, int count, float scale)
+{	while (--count >= 0)
+	{	dest [count] = scale * src [count] ;
+		} ;
+} /* s2f_array */
+
+static inline void
+i2f_array (const int *src, float *dest, int count, float scale)
+{	while (--count >= 0)
+	{	dest [count] = scale * src [count] ;
+		} ;
+} /* i2f_array */
+
+static inline void
+d2f_array (const double *src, float *dest, int count)
+{	while (--count >= 0)
+	{	dest [count] = src [count] ;
+		} ;
+} /* d2f_array */
+
 /*----------------------------------------------------------------------------------------------
 */
 
 static sf_count_t
 host_read_f2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	void		(*convert) (const float *, int, short *, float) ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	convert = (psf->add_clipping) ? f2s_clip_array : f2s_array ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFF / psf->float_max ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
 /* Fix me : Need lef2s_array */
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		f2s_array ((float*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		if (thisread < readcount)
+		convert (psf->u.fbuf, readcount, ptr + total, scale) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -462,23 +535,28 @@ host_read_f2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 
 static sf_count_t
 host_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	void		(*convert) (const float *, int, int *, float) ;
+	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	convert = (psf->add_clipping) ? f2i_clip_array : f2i_array ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFFFFFF / psf->float_max ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		f2i_array ((float*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		if (thisread < readcount)
+		convert (psf->u.fbuf, readcount, ptr + total, scale) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -486,24 +564,25 @@ host_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 
 static sf_count_t
 host_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	if (psf->float_endswap != SF_TRUE)
+	if (psf->data_endswap != SF_TRUE)
 		return psf_fread (ptr, sizeof (float), len, psf) ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		endswap_int_copy ((int*) (ptr + total), (int*) psf->buffer, thisread) ;
+		endswap_int_copy ((int*) (ptr + total), psf->u.ibuf, readcount) ;
 
-		total += thisread ;
-		if (thisread < readcount)
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -511,134 +590,144 @@ host_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 static sf_count_t
 host_read_f2d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
 /* Fix me : Need lef2d_array */
-		f2d_array ((float*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		if (thisread < readcount)
+		f2d_array (psf->u.fbuf, readcount, ptr + total) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
 } /* host_read_f2d */
 
 static sf_count_t
-host_write_s2f	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	int			bufferlen, writecount, thiswrite ;
+host_write_s2f	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+/* Erik */
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / 0x8000 ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		s2f_array (ptr + total, (float*) (psf->buffer), writecount) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		s2f_array (ptr + total, psf->u.fbuf, bufferlen, scale) ;
 
-		if (psf->has_peak)
-			float32_peak_update (psf, (float *) (psf->buffer), writecount, (int) (total / psf->sf.channels)) ;
+		if (psf->peak_info)
+			float32_peak_update (psf, psf->u.fbuf, bufferlen, total / psf->sf.channels) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float), writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* host_write_s2f */
 
 static sf_count_t
-host_write_i2f	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	int			bufferlen, writecount, thiswrite ;
+host_write_i2f	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / (8.0 * 0x10000000) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		i2f_array (ptr + total, (float*) (psf->buffer), writecount) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		i2f_array (ptr + total, psf->u.fbuf, bufferlen, scale) ;
 
-		if (psf->has_peak)
-			float32_peak_update (psf, (float *) (psf->buffer), writecount, (int) (total / psf->sf.channels)) ;
+		if (psf->peak_info)
+			float32_peak_update (psf, psf->u.fbuf, bufferlen, total / psf->sf.channels) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float) , writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float) , bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* host_write_i2f */
 
 static sf_count_t
-host_write_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
-{	int			bufferlen, writecount, thiswrite ;
+host_write_f	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	if (psf->has_peak)
+	if (psf->peak_info)
 		float32_peak_update (psf, ptr, len, 0) ;
 
-	if (psf->float_endswap != SF_TRUE)
+	if (psf->data_endswap != SF_TRUE)
 		return psf_fwrite (ptr, sizeof (float), len, psf) ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
 
-		endswap_int_copy ((int*) psf->buffer, (int*) (ptr + total), writecount) ;
+		endswap_int_copy (psf->u.ibuf, (const int*) (ptr + total), bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float), writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* host_write_f */
 
 static sf_count_t
-host_write_d2f	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
-{	int			bufferlen, writecount, thiswrite ;
+host_write_d2f	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
 
-		d2f_array (ptr + total, (float*) (psf->buffer), writecount) ;
+		d2f_array (ptr + total, psf->u.fbuf, bufferlen) ;
 
-		if (psf->has_peak)
-			float32_peak_update (psf, (float *) (psf->buffer), writecount, (int) (total / psf->sf.channels)) ;
+		if (psf->peak_info)
+			float32_peak_update (psf, psf->u.fbuf, bufferlen, total / psf->sf.channels) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float), writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
@@ -647,79 +736,30 @@ host_write_d2f	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 /*=======================================================================================
 */
 
-static void
-f2s_array (float *src, int count, short *dest)
-{	while (count)
-	{	count -- ;
-		dest [count] = lrintf (src [count]) ;
-		} ;
-} /* f2s_array */
-
-static void
-f2i_array (float *src, int count, int *dest)
-{	while (count)
-	{	count -- ;
-		dest [count] = lrintf (src [count]) ;
-		} ;
-} /* f2i_array */
-
-static void
-f2d_array (float *src, int count, double *dest)
-{	while (count)
-	{	count -- ;
-		dest [count] = src [count] ;
-		} ;
-} /* f2d_array */
-
-static void
-s2f_array (short *src, float *dest, int count)
-{	while (count)
-	{	count -- ;
-		dest [count] = src [count] ;
-		} ;
-
-} /* s2f_array */
-
-static void
-i2f_array (int *src, float *dest, int count)
-{	while (count)
-	{	count -- ;
-		dest [count] = src [count] ;
-		} ;
-} /* i2f_array */
-
-static void
-d2f_array (double *src, float *dest, int count)
-{	while (count)
-	{	count -- ;
-		dest [count] = src [count] ;
-		} ;
-} /* d2f_array */
-
-/*=======================================================================================
-*/
-
 static sf_count_t
 replace_read_f2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFF / psf->float_max ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		bf2f_array ((float *) (psf->buffer), readcount) ;
+		bf2f_array (psf->u.fbuf, bufferlen) ;
 
-		f2s_array ((float*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		if (thisread < readcount)
+		f2s_array (psf->u.fbuf, readcount, ptr + total, scale) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -727,25 +767,28 @@ replace_read_f2s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
+	scale = (psf->float_int_mult == 0) ? 1.0 : 0x7FFF / psf->float_max ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		bf2f_array ((float *) (psf->buffer), readcount) ;
+		bf2f_array (psf->u.fbuf, bufferlen) ;
 
-		f2i_array ((float*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		if (thisread < readcount)
+		f2i_array (psf->u.fbuf, readcount, ptr + total, scale) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -753,28 +796,29 @@ replace_read_f2i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
 	/* FIX THIS */
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		bf2f_array ((float *) (psf->buffer), readcount) ;
+		bf2f_array (psf->u.fbuf, bufferlen) ;
 
-		memcpy (ptr + total, psf->buffer, readcount * sizeof (float)) ;
+		memcpy (ptr + total, psf->u.fbuf, bufferlen * sizeof (float)) ;
 
-		total += thisread ;
-		if (thisread < readcount)
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
@@ -782,143 +826,152 @@ replace_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
 
 static sf_count_t
 replace_read_f2d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
-{	int			bufferlen, readcount, thisread ;
+{	int			bufferlen, readcount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	readcount = (len >= bufferlen) ? bufferlen : (int) len ;
-		thisread = psf_fread (psf->buffer, sizeof (float), readcount, psf) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		readcount = psf_fread (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, readcount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		bf2f_array ((float *) (psf->buffer), readcount) ;
+		bf2f_array (psf->u.fbuf, bufferlen) ;
 
-		f2d_array ((float*) (psf->buffer), thisread, ptr + total) ;
-		total += thisread ;
-		if (thisread < readcount)
+		f2d_array (psf->u.fbuf, readcount, ptr + total) ;
+		total += readcount ;
+		if (readcount < bufferlen)
 			break ;
-		len -= thisread ;
+		len -= readcount ;
 		} ;
 
 	return total ;
 } /* replace_read_f2d */
 
 static sf_count_t
-replace_write_s2f	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
-{	int			writecount, bufferlen, thiswrite ;
+replace_write_s2f	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / 0x8000 ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		s2f_array (ptr + total, (float*) (psf->buffer), writecount) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		s2f_array (ptr + total, psf->u.fbuf, bufferlen, scale) ;
 
-		if (psf->has_peak)
-			float32_peak_update (psf, (float *) (psf->buffer), writecount, (int) (total / psf->sf.channels)) ;
+		if (psf->peak_info)
+			float32_peak_update (psf, psf->u.fbuf, bufferlen, total / psf->sf.channels) ;
 
-		f2bf_array ((float *) (psf->buffer), writecount) ;
+		f2bf_array (psf->u.fbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float), writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* replace_write_s2f */
 
 static sf_count_t
-replace_write_i2f	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
-{	int			writecount, bufferlen, thiswrite ;
+replace_write_i2f	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
+	float		scale ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	scale = (psf->scale_int_float == 0) ? 1.0 : 1.0 / (8.0 * 0x10000000) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		i2f_array (ptr + total, (float*) (psf->buffer), writecount) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		i2f_array (ptr + total, psf->u.fbuf, bufferlen, scale) ;
 
-		if (psf->has_peak)
-			float32_peak_update (psf, (float *) (psf->buffer), writecount, (int) (total / psf->sf.channels)) ;
+		if (psf->peak_info)
+			float32_peak_update (psf, psf->u.fbuf, bufferlen, total / psf->sf.channels) ;
 
-		f2bf_array ((float *) (psf->buffer), writecount) ;
+		f2bf_array (psf->u.fbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float), writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* replace_write_i2f */
 
 static sf_count_t
-replace_write_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
-{	int			writecount, bufferlen, thiswrite ;
+replace_write_f	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
 	/* FIX THIS */
-	if (psf->has_peak)
+	if (psf->peak_info)
 		float32_peak_update (psf, ptr, len, 0) ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
 
-		memcpy (psf->buffer, ptr + total, writecount * sizeof (float)) ;
+		memcpy (psf->u.fbuf, ptr + total, bufferlen * sizeof (float)) ;
 
-		f2bf_array ((float *) (psf->buffer), writecount) ;
+		f2bf_array (psf->u.fbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float) , writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float) , bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
 } /* replace_write_f */
 
 static sf_count_t
-replace_write_d2f	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
-{	int			writecount, bufferlen, thiswrite ;
+replace_write_d2f	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
+{	int			bufferlen, writecount ;
 	sf_count_t	total = 0 ;
 
-	bufferlen = sizeof (psf->buffer) / sizeof (float) ;
+	bufferlen = ARRAY_LEN (psf->u.fbuf) ;
 
 	while (len > 0)
-	{	writecount = (len >= bufferlen) ? bufferlen : (int) len ;
-		d2f_array (ptr + total, (float*) (psf->buffer), writecount) ;
+	{	if (len < bufferlen)
+			bufferlen = (int) len ;
+		d2f_array (ptr + total, psf->u.fbuf, bufferlen) ;
 
-		if (psf->has_peak)
-			float32_peak_update (psf, (float *) (psf->buffer), writecount, (int) (total / psf->sf.channels)) ;
+		if (psf->peak_info)
+			float32_peak_update (psf, psf->u.fbuf, bufferlen, total / psf->sf.channels) ;
 
-		f2bf_array ((float *) (psf->buffer), writecount) ;
+		f2bf_array (psf->u.fbuf, bufferlen) ;
 
-		if (psf->float_endswap == SF_TRUE)
-			endswap_int_array ((int*) psf->buffer, writecount) ;
+		if (psf->data_endswap == SF_TRUE)
+			endswap_int_array (psf->u.ibuf, bufferlen) ;
 
-		thiswrite = psf_fwrite (psf->buffer, sizeof (float), writecount, psf) ;
-		total += thiswrite ;
-		if (thiswrite < writecount)
+		writecount = psf_fwrite (psf->u.fbuf, sizeof (float), bufferlen, psf) ;
+		total += writecount ;
+		if (writecount < bufferlen)
 			break ;
-		len -= thiswrite ;
+		len -= writecount ;
 		} ;
 
 	return total ;
@@ -929,24 +982,15 @@ replace_write_d2f	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 static void
 bf2f_array (float *buffer, int count)
-{	while (count)
-	{	count -- ;
-		buffer [count] = FLOAT32_READ ((unsigned char *) (buffer + count)) ;
+{	while (--count >= 0)
+	{	buffer [count] = FLOAT32_READ ((unsigned char *) (buffer + count)) ;
 		} ;
 } /* bf2f_array */
 
 static void
 f2bf_array (float *buffer, int count)
-{	while (count)
-	{	count -- ;
-		FLOAT32_WRITE (buffer [count], (unsigned char*) (buffer + count)) ;
+{	while (--count >= 0)
+	{	FLOAT32_WRITE (buffer [count], (unsigned char*) (buffer + count)) ;
 		} ;
 } /* f2bf_array */
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch
-** revision control system.
-**
-** arch-tag: b6c34917-488c-4145-9648-f4371fc4c889
-*/
