@@ -93,6 +93,16 @@ static int aiff_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
 static int aiff_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima) ;
 
 
+static inline int
+clamp_ima_step_index (int indx)
+{	if (indx < 0)
+		return 0 ;
+	if (indx >= ARRAY_LEN (ima_step_size))
+		return ARRAY_LEN (ima_step_size) - 1 ;
+
+	return indx ;
+} /* clamp_ima_step_index */
+
 /*============================================================================================
 ** IMA ADPCM Reader initialisation function.
 */
@@ -228,7 +238,6 @@ ima_reader_init (SF_PRIVATE *psf, int blockalign, int samplesperblock)
 		default :
 				psf_log_printf (psf, "ima_reader_init: bad psf->sf.format\n") ;
 				return SFE_INTERNAL ;
-				break ;
 		} ;
 
 	pima->decode_block (psf, pima) ;	/* Read first block. */
@@ -267,16 +276,9 @@ count ++ ;
 		sampledata = pima->samples + chan ;
 
 		predictor = (blockdata [0] << 8) | (blockdata [1] & 0x80) ;
-		stepindx = blockdata [1] & 0x7F ;
 
-{
-if (count < 5)
-printf ("\nchan: %d    predictor: %d    stepindx: %d (%d)\n",
-	chan, predictor, stepindx, ima_step_size [stepindx]) ;
-}
-		/* FIXME : Do this a better way. */
-		if (stepindx < 0) stepindx = 0 ;
-		else if (stepindx > 88)	stepindx = 88 ;
+		stepindx = blockdata [1] & 0x7F ;
+		stepindx = clamp_ima_step_index (stepindx) ;
 
 		/*
 		**	Pull apart the packed 4 bit samples and store them in their
@@ -295,9 +297,7 @@ printf ("\nchan: %d    predictor: %d    stepindx: %d (%d)\n",
 			bytecode = pima->samples [pima->channels * k + chan] ;
 
 			stepindx += ima_indx_adjust [bytecode] ;
-
-			if (stepindx < 0) stepindx = 0 ;
-			else if (stepindx > 88) stepindx = 88 ;
+			stepindx = clamp_ima_step_index (stepindx) ;
 
 			diff = step >> 3 ;
 			if (bytecode & 1)	diff += step >> 2 ;
@@ -306,18 +306,10 @@ printf ("\nchan: %d    predictor: %d    stepindx: %d (%d)\n",
 			if (bytecode & 8)	diff = -diff ;
 
 			predictor += diff ;
-
 			pima->samples [pima->channels * k + chan] = predictor ;
 			} ;
 		} ;
 
-if (count < 5)
-{
-	for (k = 0 ; k < 10 ; k++)
-		printf ("% 7d,", pima->samples [k]) ;
-	puts ("") ;
-}
-
 	return 1 ;
 } /* aiff_ima_decode_block */
 
@@ -326,14 +318,6 @@ aiff_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 {	int		chan, k, step, diff, vpdiff, blockindx, indx ;
 	short	bytecode, mask ;
 
-static int count = 0 ;
-if (0 && count == 0)
-{	pima->samples [0] = 0 ;
-	printf ("blocksize : %d\n", pima->blocksize) ;
-	printf ("pima->stepindx [0] : %d\n", pima->stepindx [0]) ;
-	}
-count ++ ;
-
 	/* Encode the block header. */
 	for (chan = 0 ; chan < pima->channels ; chan ++)
 	{	blockindx = chan * pima->blocksize ;
@@ -379,11 +363,8 @@ count ++ ;
 			pima->previous [chan] = -32768 ;
 
 		pima->stepindx [chan] += ima_indx_adjust [bytecode] ;
-		if (pima->stepindx [chan] < 0)
-			pima->stepindx [chan] = 0 ;
-		else if (pima->stepindx [chan] > 88)
-			pima->stepindx [chan] = 88 ;
 
+		pima->stepindx [chan] = clamp_ima_step_index (pima->stepindx [chan]) ;
 		pima->samples [k] = bytecode ;
 		} ;
 
@@ -393,9 +374,6 @@ count ++ ;
 	{	for (indx = pima->channels ; indx < pima->channels * pima->samplesperblock ; indx += 2 * pima->channels)
 		{	blockindx = chan * pima->blocksize + 2 + indx / 2 ;
 
-if (0 && count ++ < 5)
-	printf ("chan: %d    blockindx: %3d    indx: %3d\n", chan, blockindx, indx) ;
-
 			pima->block [blockindx] = pima->samples [indx] & 0x0F ;
 			pima->block [blockindx] |= (pima->samples [indx + pima->channels] << 4) & 0xF0 ;
 			} ;
@@ -437,10 +415,8 @@ wav_w64_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 			current -= 0x10000 ;
 
 		stepindx [chan] = pima->block [chan*4+2] ;
-		if (stepindx [chan] < 0)
-			stepindx [chan] = 0 ;
-		else if (stepindx [chan] > 88)
-			stepindx [chan] = 88 ;
+		stepindx [chan] = clamp_ima_step_index (stepindx [chan]) ;
+
 
 		if (pima->block [chan*4+3] != 0)
 			psf_log_printf (psf, "IMA ADPCM synchronisation error.\n") ;
@@ -498,11 +474,7 @@ wav_w64_ima_decode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 			current = -32768 ;
 
 		stepindx [chan] += ima_indx_adjust [bytecode] ;
-
-		if (stepindx [chan] < 0)
-			stepindx [chan] = 0 ;
-		else if (stepindx [chan] > 88)
-			stepindx [chan] = 88 ;
+		stepindx [chan] = clamp_ima_step_index (stepindx [chan]) ;
 
 		pima->samples [k] = current ;
 		} ;
@@ -562,10 +534,7 @@ wav_w64_ima_encode_block (SF_PRIVATE *psf, IMA_ADPCM_PRIVATE *pima)
 			pima->previous [chan] = -32768 ;
 
 		pima->stepindx [chan] += ima_indx_adjust [bytecode] ;
-		if (pima->stepindx [chan] < 0)
-			pima->stepindx [chan] = 0 ;
-		else if (pima->stepindx [chan] > 88)
-			pima->stepindx [chan] = 88 ;
+		pima->stepindx [chan] = clamp_ima_step_index (pima->stepindx [chan]) ;
 
 		pima->samples [k] = bytecode ;
 		} ;
@@ -826,7 +795,6 @@ ima_writer_init (SF_PRIVATE *psf, int blockalign)
 		default :
 				psf_log_printf (psf, "ima_reader_init: bad psf->sf.format\n") ;
 				return SFE_INTERNAL ;
-				break ;
 		} ;
 
 	psf->write_short	= ima_write_s ;
