@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2005 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 1999-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -21,10 +21,10 @@
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
+#include	<math.h>
 
 #include	"sndfile.h"
 #include	"sfendian.h"
-#include	"float_cast.h"
 #include	"common.h"
 #include	"wav_w64.h"
 
@@ -132,8 +132,8 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	unsigned int	pmssize ;
 	int				count ;
 
-	if (psf->fdata != NULL)
-	{	psf_log_printf (psf, "*** psf->fdata is not NULL.\n") ;
+	if (psf->codec_data != NULL)
+	{	psf_log_printf (psf, "*** psf->codec_data is not NULL.\n") ;
 		return SFE_INTERNAL ;
 		} ;
 
@@ -142,9 +142,9 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 
 	pmssize = sizeof (MSADPCM_PRIVATE) + blockalign + 3 * psf->sf.channels * samplesperblock ;
 
-	if (! (psf->fdata = malloc (pmssize)))
+	if (! (psf->codec_data = malloc (pmssize)))
 		return SFE_MALLOC_FAILED ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 	memset (pms, 0, pmssize) ;
 
 	pms->samples	= pms->dummydata ;
@@ -154,6 +154,11 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 	pms->blocksize	= blockalign ;
 	pms->samplesperblock = samplesperblock ;
 
+	if (pms->blocksize == 0)
+	{	psf_log_printf (psf, "*** Error : pms->blocksize should not be zero.\n") ;
+		return SFE_INTERNAL ;
+		} ;
+
 	if (psf->mode == SFM_READ)
 	{	pms->dataremaining	 = psf->datalength ;
 
@@ -164,7 +169,9 @@ wav_w64_msadpcm_init	(SF_PRIVATE *psf, int blockalign, int samplesperblock)
 
 		count = 2 * (pms->blocksize - 6 * pms->channels) / pms->channels ;
 		if (pms->samplesperblock != count)
-			psf_log_printf (psf, "*** Warning : samplesperblock shoud be %d.\n", count) ;
+		{	psf_log_printf (psf, "*** Error : samplesperblock should be %d.\n", count) ;
+			return SFE_INTERNAL ;
+			} ;
 
 		psf->sf.frames = (psf->datalength / pms->blocksize) * pms->samplesperblock ;
 
@@ -336,9 +343,9 @@ msadpcm_read_s	(SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int			readcount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	while (len > 0)
 	{	readcount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -361,9 +368,9 @@ msadpcm_read_i	(SF_PRIVATE *psf, int *ptr, sf_count_t len)
 	int			k, bufferlen, readcount = 0, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	sptr = psf->u.sbuf ;
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -388,9 +395,9 @@ msadpcm_read_f	(SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? 1.0 / ((float) 0x8000) : 1.0 ;
 	sptr = psf->u.sbuf ;
@@ -418,9 +425,9 @@ msadpcm_read_d	(SF_PRIVATE *psf, double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? 1.0 / ((double) 0x8000) : 1.0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	sptr = psf->u.sbuf ;
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -442,9 +449,9 @@ msadpcm_seek	(SF_PRIVATE *psf, int mode, sf_count_t offset)
 {	MSADPCM_PRIVATE *pms ;
 	int			newblock, newsample ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	if (psf->datalength < 0 || psf->dataoffset < 0)
 	{	psf->error = SFE_BAD_SEEK ;
@@ -646,9 +653,9 @@ msadpcm_write_s	(SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 	int			writecount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	while (len > 0)
 	{	writecount = (len > 0x10000000) ? 0x10000000 : (int) len ;
@@ -671,9 +678,9 @@ msadpcm_write_i	(SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 	int			k, bufferlen, writecount, count ;
 	sf_count_t	total = 0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	sptr = psf->u.sbuf ;
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -698,9 +705,9 @@ msadpcm_write_f	(SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	normfact = (psf->norm_float == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
 
@@ -729,9 +736,9 @@ msadpcm_write_d	(SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 
 	normfact = (psf->norm_double == SF_TRUE) ? (1.0 * 0x7FFF) : 1.0 ;
 
-	if (! psf->fdata)
+	if (! psf->codec_data)
 		return 0 ;
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	sptr = psf->u.sbuf ;
 	bufferlen = ARRAY_LEN (psf->u.sbuf) ;
@@ -755,7 +762,7 @@ static int
 msadpcm_close	(SF_PRIVATE *psf)
 {	MSADPCM_PRIVATE *pms ;
 
-	pms = (MSADPCM_PRIVATE*) psf->fdata ;
+	pms = (MSADPCM_PRIVATE*) psf->codec_data ;
 
 	if (psf->mode == SFM_WRITE)
 	{	/*  Now we know static int for certain the length of the file we can
@@ -825,10 +832,4 @@ choose_predictor (unsigned int channels, short *data, int *block_pred, int *idel
 
 	return ;
 } /* choose_predictor */
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch 
-** revision control system.
-**
-** arch-tag: a98908a3-5305-4935-872b-77d6a86c330f
-*/
+
