@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2009 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,18 +16,18 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include "config.h"
+#include "sfconfig.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <string.h>
 #include <ctype.h>
+#include <math.h>
 
 #include "sndfile.h"
 #include "sfendian.h"
 #include "common.h"
-#include "float_cast.h"
 
 /*------------------------------------------------------------------------------
 */
@@ -76,10 +76,10 @@ static sf_count_t sds_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
 static sf_count_t sds_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
 static sf_count_t sds_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
 
-static sf_count_t sds_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len) ;
-static sf_count_t sds_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len) ;
-static sf_count_t sds_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len) ;
-static sf_count_t sds_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len) ;
+static sf_count_t sds_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len) ;
+static sf_count_t sds_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len) ;
+static sf_count_t sds_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len) ;
+static sf_count_t sds_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len) ;
 
 static sf_count_t sds_seek (SF_PRIVATE *psf, int mode, sf_count_t offset) ;
 
@@ -93,7 +93,7 @@ static int sds_2byte_write (SF_PRIVATE *psf, SDS_PRIVATE *psds) ;
 static int sds_3byte_write (SF_PRIVATE *psf, SDS_PRIVATE *psds) ;
 static int sds_4byte_write (SF_PRIVATE *psf, SDS_PRIVATE *psds) ;
 
-static int sds_write (SF_PRIVATE *psf, SDS_PRIVATE *psds, int *iptr, int writecount) ;
+static int sds_write (SF_PRIVATE *psf, SDS_PRIVATE *psds, const int *iptr, int writecount) ;
 
 /*------------------------------------------------------------------------------
 ** Public function.
@@ -102,25 +102,23 @@ static int sds_write (SF_PRIVATE *psf, SDS_PRIVATE *psds, int *iptr, int writeco
 int
 sds_open	(SF_PRIVATE *psf)
 {	SDS_PRIVATE	*psds ;
-	int			subformat, error = 0 ;
+	int			error = 0 ;
 
 	/* Hmmmm, need this here to pass update_header_test. */
 	psf->sf.frames = 0 ;
 
 	if (! (psds = calloc (1, sizeof (SDS_PRIVATE))))
 		return SFE_MALLOC_FAILED ;
-	psf->fdata = psds ;
+	psf->codec_data = psds ;
 
 	if (psf->mode == SFM_READ || (psf->mode == SFM_RDWR && psf->filelength > 0))
 	{	if ((error = sds_read_header (psf, psds)))
 			return error ;
 		} ;
 
-	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_SDS)
+	if ((SF_CONTAINER (psf->sf.format)) != SF_FORMAT_SDS)
 		return	SFE_BAD_OPEN_FORMAT ;
 
-	subformat = psf->sf.format & SF_FORMAT_SUBMASK ;
-
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
 	{	if (sds_write_header (psf, SF_FALSE))
 			return psf->error ;
@@ -134,7 +132,7 @@ sds_open	(SF_PRIVATE *psf)
 		return error ;
 
 	psf->seek = sds_seek ;
-	psf->close = sds_close ;
+	psf->container_close = sds_close ;
 
 	psf->blockwidth = 0 ;
 
@@ -150,8 +148,8 @@ sds_close	(SF_PRIVATE *psf)
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
 	{	SDS_PRIVATE *psds ;
 
-		if ((psds = (SDS_PRIVATE *) psf->fdata) == NULL)
-		{	psf_log_printf (psf, "*** Bad psf->fdata ptr.\n") ;
+		if ((psds = (SDS_PRIVATE *) psf->codec_data) == NULL)
+		{	psf_log_printf (psf, "*** Bad psf->codec_data ptr.\n") ;
 			return SFE_INTERNAL ;
 			} ;
 
@@ -328,8 +326,8 @@ sds_write_header (SF_PRIVATE *psf, int calc_length)
 	int samp_period, data_length, sustain_loop_start, sustain_loop_end ;
 	unsigned char loop_type = 0 ;
 
-	if ((psds = (SDS_PRIVATE *) psf->fdata) == NULL)
-	{	psf_log_printf (psf, "*** Bad psf->fdata ptr.\n") ;
+	if ((psds = (SDS_PRIVATE *) psf->codec_data) == NULL)
+	{	psf_log_printf (psf, "*** Bad psf->codec_data ptr.\n") ;
 		return SFE_INTERNAL ;
 		} ;
 
@@ -362,7 +360,7 @@ sds_write_header (SF_PRIVATE *psf, int calc_length)
 
 	psf_binheader_writef (psf, "E211", 0xF07E, 0, 1) ;
 
-	switch (psf->sf.format & SF_FORMAT_SUBMASK)
+	switch (SF_CODEC (psf->sf.format))
 	{	case SF_FORMAT_PCM_S8 :
 				psds->bitwidth = 8 ;
 				break ;
@@ -545,9 +543,9 @@ sds_read_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 	int			k, bufferlen, readcount, count ;
 	sf_count_t	total = 0 ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	iptr = psf->u.ibuf ;
 	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
@@ -568,9 +566,9 @@ sds_read_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 {	SDS_PRIVATE *psds ;
 	int			total ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	total = sds_read (psf, psds, ptr, len) ;
 
@@ -585,9 +583,9 @@ sds_read_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	if (psf->norm_float == SF_TRUE)
 		normfact = 1.0 / 0x80000000 ;
@@ -616,9 +614,9 @@ sds_read_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	if (psf->norm_double == SF_TRUE)
 		normfact = 1.0 / 0x80000000 ;
@@ -672,19 +670,19 @@ sds_seek (SF_PRIVATE *psf, int mode, sf_count_t seek_from_start)
 	sf_count_t	file_offset ;
 	int			newblock, newsample ;
 
-	if ((psds = psf->fdata) == NULL)
+	if ((psds = psf->codec_data) == NULL)
 	{	psf->error = SFE_INTERNAL ;
-		return SF_SEEK_ERROR ;
+		return PSF_SEEK_ERROR ;
 		} ;
 
 	if (psf->datalength < 0 || psf->dataoffset < 0)
 	{	psf->error = SFE_BAD_SEEK ;
-		return SF_SEEK_ERROR ;
+		return PSF_SEEK_ERROR ;
 		} ;
 
 	if (seek_from_start < 0 || seek_from_start > psf->sf.frames)
 	{	psf->error = SFE_BAD_SEEK ;
-		return SF_SEEK_ERROR ;
+		return PSF_SEEK_ERROR ;
 		} ;
 
 	if (mode == SFM_READ && psds->write_count > 0)
@@ -697,14 +695,14 @@ sds_seek (SF_PRIVATE *psf, int mode, sf_count_t seek_from_start)
 	{	case SFM_READ :
 			if (newblock > psds->total_blocks)
 			{	psf->error = SFE_BAD_SEEK ;
-				return SF_SEEK_ERROR ;
+				return PSF_SEEK_ERROR ;
 				} ;
 
 			file_offset = psf->dataoffset + newblock * SDS_BLOCK_SIZE ;
 
 			if (psf_fseek (psf, file_offset, SEEK_SET) != file_offset)
 			{	psf->error = SFE_SEEK_FAILED ;
-				return SF_SEEK_ERROR ;
+				return PSF_SEEK_ERROR ;
 				} ;
 
 			psds->read_block = newblock ;
@@ -715,14 +713,14 @@ sds_seek (SF_PRIVATE *psf, int mode, sf_count_t seek_from_start)
 		case SFM_WRITE :
 			if (newblock > psds->total_blocks)
 			{	psf->error = SFE_BAD_SEEK ;
-				return SF_SEEK_ERROR ;
+				return PSF_SEEK_ERROR ;
 				} ;
 
 			file_offset = psf->dataoffset + newblock * SDS_BLOCK_SIZE ;
 
 			if (psf_fseek (psf, file_offset, SEEK_SET) != file_offset)
 			{	psf->error = SFE_SEEK_FAILED ;
-				return SF_SEEK_ERROR ;
+				return PSF_SEEK_ERROR ;
 				} ;
 
 			psds->write_block = newblock ;
@@ -732,7 +730,7 @@ sds_seek (SF_PRIVATE *psf, int mode, sf_count_t seek_from_start)
 
 		default :
 			psf->error = SFE_BAD_SEEK ;
-			return SF_SEEK_ERROR ;
+			return PSF_SEEK_ERROR ;
 			break ;
 		} ;
 
@@ -866,15 +864,15 @@ sds_4byte_write (SF_PRIVATE *psf, SDS_PRIVATE *psds)
 } /* sds_4byte_write */
 
 static sf_count_t
-sds_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
+sds_write_s (SF_PRIVATE *psf, const short *ptr, sf_count_t len)
 {	SDS_PRIVATE	*psds ;
 	int			*iptr ;
 	int			k, bufferlen, writecount, count ;
 	sf_count_t	total = 0 ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	iptr = psf->u.ibuf ;
 	bufferlen = ARRAY_LEN (psf->u.ibuf) ;
@@ -891,13 +889,13 @@ sds_write_s (SF_PRIVATE *psf, short *ptr, sf_count_t len)
 } /* sds_write_s */
 
 static sf_count_t
-sds_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
+sds_write_i (SF_PRIVATE *psf, const int *ptr, sf_count_t len)
 {	SDS_PRIVATE *psds ;
 	int			total ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	total = sds_write (psf, psds, ptr, len) ;
 
@@ -905,16 +903,16 @@ sds_write_i (SF_PRIVATE *psf, int *ptr, sf_count_t len)
 } /* sds_write_i */
 
 static sf_count_t
-sds_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
+sds_write_f (SF_PRIVATE *psf, const float *ptr, sf_count_t len)
 {	SDS_PRIVATE	*psds ;
 	int			*iptr ;
 	int			k, bufferlen, writecount, count ;
 	sf_count_t	total = 0 ;
 	float		normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	if (psf->norm_float == SF_TRUE)
 		normfact = 1.0 * 0x80000000 ;
@@ -936,16 +934,16 @@ sds_write_f (SF_PRIVATE *psf, float *ptr, sf_count_t len)
 } /* sds_write_f */
 
 static sf_count_t
-sds_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
+sds_write_d (SF_PRIVATE *psf, const double *ptr, sf_count_t len)
 {	SDS_PRIVATE	*psds ;
 	int			*iptr ;
 	int			k, bufferlen, writecount, count ;
 	sf_count_t	total = 0 ;
 	double		normfact ;
 
-	if (psf->fdata == NULL)
+	if (psf->codec_data == NULL)
 		return 0 ;
-	psds = (SDS_PRIVATE*) psf->fdata ;
+	psds = (SDS_PRIVATE*) psf->codec_data ;
 
 	if (psf->norm_double == SF_TRUE)
 		normfact = 1.0 * 0x80000000 ;
@@ -967,7 +965,7 @@ sds_write_d (SF_PRIVATE *psf, double *ptr, sf_count_t len)
 } /* sds_write_d */
 
 static int
-sds_write (SF_PRIVATE *psf, SDS_PRIVATE *psds, int *ptr, int len)
+sds_write (SF_PRIVATE *psf, SDS_PRIVATE *psds, const int *ptr, int len)
 {	int	count, total = 0 ;
 
 	while (total < len)
@@ -986,10 +984,3 @@ sds_write (SF_PRIVATE *psf, SDS_PRIVATE *psds, int *ptr, int len)
 	return total ;
 } /* sds_write */
 
-/*
-** Do not edit or modify anything in this comment block.
-** The arch-tag line is a file identity tag for the GNU Arch
-** revision control system.
-**
-** arch-tag: d5d26aa3-368c-4ca6-bb85-377e5a2578cc
-*/
