@@ -112,7 +112,7 @@ ErrorStruct SndfileErrors [] =
 	{	SFE_RDWR_POSITION		, "Error on RDWR position (cryptic)." },
 	{	SFE_RDWR_BAD_HEADER		, "Error : Cannot open file in read/write mode due to string data in header." },
 	{	SFE_CMD_HAS_DATA		, "Error : Command fails because file already has audio data." },
-	
+
 	{	SFE_STR_NO_SUPPORT		, "Error : File type does not support string data." },
 	{	SFE_STR_NOT_WRITE		, "Error : Trying to set a string when file is not in write mode." },
 	{	SFE_STR_MAX_DATA		, "Error : Maximum string data storage reached." },
@@ -317,9 +317,9 @@ sf_open	(const char *path, int mode, SF_INFO *sfinfo)
 	copy_filename (psf, path) ;
 
 	if (strcmp (path, "-") == 0)
-		psf_set_stdio (psf, mode) ;
+		psf->error = psf_set_stdio (psf, mode) ;
 	else
-		psf_fopen (psf, path, mode) ;
+		psf->error = psf_fopen (psf, path, mode) ;
 
 	return psf_open_file (psf, mode, sfinfo) ;
 } /* sf_open */
@@ -357,19 +357,19 @@ sf_open_virtual	(SF_VIRTUAL_IO *sfvirtual, int mode, SF_INFO *sfinfo, void *user
 	/* Make sure we have a valid set ot virtual pointers. */
 	if (sfvirtual->get_filelen == NULL || sfvirtual->seek == NULL || sfvirtual->tell == NULL)
 	{	sf_errno = SFE_BAD_VIRTUAL_IO ;
-		LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "Bad vio_get_filelen / vio_seek / vio_tell in SF_VIRTUAL_IO struct.\n") ;
+		snprintf (sf_logbuffer, sizeof (sf_logbuffer), "Bad vio_get_filelen / vio_seek / vio_tell in SF_VIRTUAL_IO struct.\n") ;
 		return NULL ;
 		} ;
 
 	if ((mode == SFM_READ || mode == SFM_RDWR) && sfvirtual->read == NULL)
 	{	sf_errno = SFE_BAD_VIRTUAL_IO ;
-		LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "Bad vio_read in SF_VIRTUAL_IO struct.\n") ;
+		snprintf (sf_logbuffer, sizeof (sf_logbuffer), "Bad vio_read in SF_VIRTUAL_IO struct.\n") ;
 		return NULL ;
 		} ;
 
 	if ((mode == SFM_WRITE || mode == SFM_RDWR) && sfvirtual->write == NULL)
 	{	sf_errno = SFE_BAD_VIRTUAL_IO ;
-		LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "Bad vio_write in SF_VIRTUAL_IO struct.\n") ;
+		snprintf (sf_logbuffer, sizeof (sf_logbuffer), "Bad vio_write in SF_VIRTUAL_IO struct.\n") ;
 		return NULL ;
 		} ;
 
@@ -517,7 +517,7 @@ sf_error_str (SNDFILE *sndfile, char *str, size_t maxlen)
 		errnum = psf->error ;
 		} ;
 
-	LSF_SNPRINTF (str, maxlen, "%s", sf_error_number (errnum)) ;
+	snprintf (str, maxlen, "%s", sf_error_number (errnum)) ;
 
 	return SFE_NO_ERROR ;
 } /* sf_error_str */
@@ -537,7 +537,7 @@ sf_format_check	(const SF_INFO *info)
 	** Return 0 on failure, 1 ons success.
 	*/
 
-	if (info->channels < 1 || info->channels > 256)
+	if (info->channels < 1 || info->channels > SF_MAX_CHANNELS)
 		return 0 ;
 
 	if (info->samplerate < 0)
@@ -811,23 +811,37 @@ sf_format_check	(const SF_INFO *info)
 	return 0 ;
 } /* sf_format_check */
 
+/*------------------------------------------------------------------------------
+*/
+
+const char *
+sf_version_string (void)
+{
+#if	ENABLE_EXPERIMENTAL_CODE
+	return PACKAGE_NAME "-" PACKAGE_VERSION "-exp" ;
+#else
+	return PACKAGE_NAME "-" PACKAGE_VERSION ;
+#endif
+}
+
+
 /*------------------------------------------------------------------------------
 */
 
 int
 sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
-{	SF_PRIVATE 	*psf = NULL ;
+{	SF_PRIVATE *psf = (SF_PRIVATE *) sndfile ;
 	int old_value ;
 
 	/* This set of commands do not need the sndfile parameter. */
 	switch (command)
 	{	case SFC_GET_LIB_VERSION :
 			if (data == NULL)
-				return (psf->error = SFE_BAD_COMMAND_PARAM) ;
-			if (ENABLE_EXPERIMENTAL_CODE)
-				LSF_SNPRINTF (data, datasize, "%s-%s-exp", PACKAGE_NAME, PACKAGE_VERSION) ;
-			else
-				LSF_SNPRINTF (data, datasize, "%s-%s", PACKAGE_NAME, PACKAGE_VERSION) ;
+			{	if (psf)
+					psf->error = SFE_BAD_COMMAND_PARAM ;
+				return SFE_BAD_COMMAND_PARAM ;
+				} ;
+			snprintf (data, datasize, "%s", sf_version_string ()) ;
 			return strlen (data) ;
 
 		case SFC_GET_SIMPLE_FORMAT_COUNT :
@@ -872,7 +886,7 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 	if (sndfile == NULL && command == SFC_GET_LOG_INFO)
 	{	if (data == NULL)
 			return (psf->error = SFE_BAD_COMMAND_PARAM) ;
-		LSF_SNPRINTF (data, datasize, "%s", sf_logbuffer) ;
+		snprintf (data, datasize, "%s", sf_logbuffer) ;
 		return strlen (data) ;
 		} ;
 
@@ -957,7 +971,7 @@ sf_command	(SNDFILE *sndfile, int command, void *data, int datasize)
 		case SFC_GET_LOG_INFO :
 			if (data == NULL)
 				return SFE_BAD_COMMAND_PARAM ;
-			LSF_SNPRINTF (data, datasize, "%s", psf->logbuffer) ;
+			snprintf (data, datasize, "%s", psf->logbuffer) ;
 			break ;
 
 		case SFC_CALC_SIGNAL_MAX :
@@ -2139,9 +2153,12 @@ sf_writef_double	(SNDFILE *sndfile, const double *ptr, sf_count_t frames)
 
 static int
 try_resource_fork (SF_PRIVATE * psf, int mode)
-{
+{	int old_error = psf->error ;
+
 	if (psf_open_rsrc (psf, mode) != 0)
+	{	psf->error = old_error ;
 		return 0 ;
+		} ;
 
 	/* More checking here. */
 	psf_log_printf (psf, "Resource fork : %s\n", psf->rsrcpath) ;
@@ -2155,9 +2172,6 @@ format_from_extension (SF_PRIVATE *psf)
 	char buffer [16] ;
 	int format = 0 ;
 
-	if (psf->filename == NULL)
-		return 0 ;
-
 	if ((cptr = strrchr (psf->filename, '.')) == NULL)
 		return 0 ;
 
@@ -2367,7 +2381,7 @@ validate_psf (SF_PRIVATE *psf)
 
 static void
 save_header_info (SF_PRIVATE *psf)
-{	LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "%s", psf->logbuffer) ;
+{	snprintf (sf_logbuffer, sizeof (sf_logbuffer), "%s", psf->logbuffer) ;
 } /* save_header_info */
 
 static void
@@ -2375,16 +2389,16 @@ copy_filename (SF_PRIVATE *psf, const char *path)
 {	const char *ccptr ;
 	char *cptr ;
 
-	LSF_SNPRINTF (psf->filepath, sizeof (psf->filepath), "%s", path) ;
+	snprintf (psf->filepath, sizeof (psf->filepath), "%s", path) ;
 	if ((ccptr = strrchr (path, '/')) || (ccptr = strrchr (path, '\\')))
 		ccptr ++ ;
 	else
 		ccptr = path ;
 
-	LSF_SNPRINTF (psf->filename, sizeof (psf->filename), "%s", ccptr) ;
+	snprintf (psf->filename, sizeof (psf->filename), "%s", ccptr) ;
 
 	/* Now grab the directory. */
-	LSF_SNPRINTF (psf->directory, sizeof (psf->directory), "%s", path) ;
+	snprintf (psf->directory, sizeof (psf->directory), "%s", path) ;
 	if ((cptr = strrchr (psf->directory, '/')) || (cptr = strrchr (psf->directory, '\\')))
 		cptr [1] = 0 ;
 	else
@@ -2774,8 +2788,8 @@ error_exit :
 	sf_errno = error ;
 
 	if (error == SFE_SYSTEM)
-		LSF_SNPRINTF (sf_syserr, sizeof (sf_syserr), "%s", psf->syserr) ;
-	LSF_SNPRINTF (sf_logbuffer, sizeof (sf_logbuffer), "%s", psf->logbuffer) ;
+		snprintf (sf_syserr, sizeof (sf_syserr), "%s", psf->syserr) ;
+	snprintf (sf_logbuffer, sizeof (sf_logbuffer), "%s", psf->logbuffer) ;
 
 	switch (error)
 	{	case SF_ERR_SYSTEM :
